# SPDX-FileCopyrightText: 2023-present Fynn Freyer <fynn.freyer@googlemail.com>
#
# SPDX-License-Identifier: AGPL-3.0-or-later

__app_name__ = "ai-chatter"
__description__ = "Toolkit for working with ChatGPT."
__version__ = "0.0.1a0"
__author__ = "Fynn Freyer"
